
#include "CIRKSettingsForm.h"
#include "irk.h"
#include <qcombobox.h>
#include <qlistview.h>
#include <qspinbox.h>
#include <stdio.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtextbrowser.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <ctype.h>

CIRKSettingsForm::CIRKSettingsForm(CGIRK *parent, const char *name, bool modal)
  : CIRKSettingsFormBase(NULL, name, modal)
{
	m_pIRK=parent;
	m_pCurrentShiftListItem=m_pCurrentNumListItem=m_pCurrentCmdListItem=m_pCurrentPunctListItem=m_pCurrentFastListItem=NULL;
	m_pKeyTarget=NULL;
	m_pKeyTrigger=NULL;
#if !POCKETOP_BUILD
	connect(mKeyboard,SIGNAL(activated(int)),SLOT(changedKeyboard(int)));
#else
	setCaption( "Pocketop Settings" );
	mDefaultOff->setText("Pocketop Off");
	mDefaultOn->setText("Pocketop On");
	delete mKeyboard;
	delete mNewKeyboard;
#if X86
	mAboutText->setSource("/home/cg2/Settings/pocketopAbout.html");
#else
	mAboutText->setSource("/home/root/Settings/pocketopAbout.html");
#endif
#endif
	connect(mShiftKeyListView,SIGNAL(selectionChanged(QListViewItem*)),SLOT(changedShiftListSelection(QListViewItem*)));
	connect(mNumKeyListView,SIGNAL(selectionChanged(QListViewItem*)),SLOT(changedNumListSelection(QListViewItem*)));
	connect(mCmdKeyListView,SIGNAL(selectionChanged(QListViewItem*)),SLOT(changedCmdListSelection(QListViewItem*)));
	connect(mPunctKeyListView,SIGNAL(selectionChanged(QListViewItem*)),SLOT(changedPunctListSelection(QListViewItem*)));
	connect(mFastKeyListView,SIGNAL(selectionChanged(QListViewItem*)),SLOT(changedFastListSelection(QListViewItem*)));
	connect(mKeyShifted,SIGNAL(textChanged(const QString&)),SLOT(changedShiftText(const QString&)));
	connect(mKeyNum,SIGNAL(textChanged(const QString&)),SLOT(changedNumText(const QString&)));
	connect(mKeyCmd,SIGNAL(textChanged(const QString&)),SLOT(changedCmdText(const QString&)));
	connect(mKeyPunct,SIGNAL(textChanged(const QString&)),SLOT(changedPunctText(const QString&)));
	connect(mKeyFast,SIGNAL(textChanged(const QString&)),SLOT(changedFastText(const QString&)));
	connect(mNewShift,SIGNAL(clicked()),SLOT(newShift()));
	connect(mNewNum,SIGNAL(clicked()),SLOT(newNum()));
	connect(mNewCmd,SIGNAL(clicked()),SLOT(newCmd()));
	connect(mNewPunct,SIGNAL(clicked()),SLOT(newPunct()));
	connect(mNewFast,SIGNAL(clicked()),SLOT(newFast()));
	connect(mDeleteShift,SIGNAL(clicked()),SLOT(deleteShift()));
	connect(mDeleteNum,SIGNAL(clicked()),SLOT(deleteNum()));
	connect(mDeleteCmd,SIGNAL(clicked()),SLOT(deleteCmd()));
	connect(mDeletePunct,SIGNAL(clicked()),SLOT(deletePunct()));
	connect(mDeleteFast,SIGNAL(clicked()),SLOT(deleteFast()));
	connect(mShiftKeyButton,SIGNAL(clicked()),SLOT(selectShiftKey()));
	connect(mNumKeyButton,SIGNAL(clicked()),SLOT(selectNumKey()));
	connect(mCmdKeyButton,SIGNAL(clicked()),SLOT(selectCmdKey()));
	connect(mPunctKeyButton,SIGNAL(clicked()),SLOT(selectPunctKey()));
	connect(mFastKeyButton,SIGNAL(clicked()),SLOT(selectFastKey()));
	connect(mRepeatEnable,SIGNAL(toggled(bool)),SLOT(enableDisableRepeat(bool)));
}

CIRKSettingsForm::~CIRKSettingsForm()
{
}

void CIRKSettingsForm::changedShiftText(const QString &s)
{
	if(mKeyUnShifted->text().length())
	{
		if(m_pCurrentShiftListItem)
		{
			m_pCurrentShiftListItem->setText(0,mKeyUnShifted->text());
			m_pCurrentShiftListItem->setText(1,s);
		}else{
			m_pCurrentShiftListItem=new QListViewItem(mShiftKeyListView,mKeyUnShifted->text(),s);
			mShiftKeyListView->setCurrentItem(m_pCurrentShiftListItem);
		}
	}
}

void CIRKSettingsForm::newShift()
{
	m_pCurrentShiftListItem=NULL;
	mKeyUnShifted->setText("");
	mKeyShifted->setText("");
}

void CIRKSettingsForm::deleteShift()
{
	mKeyUnShifted->setText("");
	mKeyShifted->setText("");
	if(m_pCurrentShiftListItem)
	{
		delete m_pCurrentShiftListItem;
		m_pCurrentShiftListItem=NULL;
	}
}

void CIRKSettingsForm::changedFastText(const QString &s)
{
	if(mKeyUnFast->text().length())
	{
		if(m_pCurrentFastListItem)
		{
			m_pCurrentFastListItem->setText(0,mKeyUnFast->text());
			m_pCurrentFastListItem->setText(1,s);
		}else{
			m_pCurrentFastListItem=new QListViewItem(mFastKeyListView,mKeyUnFast->text(),s);
			mFastKeyListView->setCurrentItem(m_pCurrentFastListItem);
		}
	}
}

void CIRKSettingsForm::newFast()
{
	m_pCurrentFastListItem=NULL;
	mKeyUnFast->setText("");
	mKeyFast->setText("");
}

void CIRKSettingsForm::deleteFast()
{
	mKeyUnFast->setText("");
	mKeyFast->setText("");
	if(m_pCurrentFastListItem)
	{
		delete m_pCurrentFastListItem;
		m_pCurrentFastListItem=NULL;
	}
}


void CIRKSettingsForm::changedNumText(const QString &s)
{
	if(mKeyUnNum->text().length())
	{
		if(m_pCurrentNumListItem)
		{
			m_pCurrentNumListItem->setText(0,mKeyUnNum->text());
			m_pCurrentNumListItem->setText(1,s);
		}else{
			m_pCurrentNumListItem=new QListViewItem(mNumKeyListView,mKeyUnNum->text(),s);
			mNumKeyListView->setCurrentItem(m_pCurrentNumListItem);
		}
	}
}

void CIRKSettingsForm::newNum()
{
	m_pCurrentNumListItem=NULL;
	mKeyUnNum->setText("");
	mKeyNum->setText("");
}

void CIRKSettingsForm::deleteNum()
{
	mKeyUnNum->setText("");
	mKeyNum->setText("");
	if(m_pCurrentNumListItem)
	{
		delete m_pCurrentNumListItem;
		m_pCurrentNumListItem=NULL;
	}
}
void CIRKSettingsForm::changedCmdText(const QString &s)
{
	if(mKeyUnCmd->text().length())
	{
		if(m_pCurrentCmdListItem)
		{
			m_pCurrentCmdListItem->setText(0,mKeyUnCmd->text());
			m_pCurrentCmdListItem->setText(1,s);
		}else{
			m_pCurrentCmdListItem=new QListViewItem(mCmdKeyListView,mKeyUnCmd->text(),s);
			mCmdKeyListView->setCurrentItem(m_pCurrentCmdListItem);
		}
	}
}

void CIRKSettingsForm::newCmd()
{
	m_pCurrentCmdListItem=NULL;
	mKeyUnCmd->setText("");
	mKeyCmd->setText("");
}

void CIRKSettingsForm::deleteCmd()
{
	mKeyUnCmd->setText("");
	mKeyCmd->setText("");
	if(m_pCurrentCmdListItem)
	{
		delete m_pCurrentCmdListItem;
		m_pCurrentCmdListItem=NULL;
	}
}
void CIRKSettingsForm::changedPunctText(const QString &s)
{
	if(mKeyUnPunct->text().length())
	{
		if(m_pCurrentPunctListItem)
		{
			m_pCurrentPunctListItem->setText(0,mKeyUnPunct->text());
			m_pCurrentPunctListItem->setText(1,s);
		}else{
			m_pCurrentPunctListItem=new QListViewItem(mPunctKeyListView,mKeyUnPunct->text(),s);
			mPunctKeyListView->setCurrentItem(m_pCurrentPunctListItem);
		}
	}
}

void CIRKSettingsForm::newPunct()
{
	m_pCurrentPunctListItem=NULL;
	mKeyUnPunct->setText("");
	mKeyPunct->setText("");
}

void CIRKSettingsForm::deletePunct()
{
	mKeyUnPunct->setText("");
	mKeyPunct->setText("");
	if(m_pCurrentPunctListItem)
	{
		delete m_pCurrentPunctListItem;
		m_pCurrentPunctListItem=NULL;
	}
}


void CIRKSettingsForm::changedShiftListSelection(QListViewItem *i)
{
	m_pCurrentShiftListItem=i;
	mKeyUnShifted->setText(i->text(0));
	mKeyShifted->setText(i->text(1));
}

void CIRKSettingsForm::changedNumListSelection(QListViewItem *i)
{
	m_pCurrentNumListItem=i;
	mKeyUnNum->setText(i->text(0));
	mKeyNum->setText(i->text(1));
}

void CIRKSettingsForm::changedCmdListSelection(QListViewItem *i)
{
	m_pCurrentCmdListItem=i;
	mKeyUnCmd->setText(i->text(0));
	mKeyCmd->setText(i->text(1));
}

void CIRKSettingsForm::changedPunctListSelection(QListViewItem *i)
{
	m_pCurrentPunctListItem=i;
	mKeyUnPunct->setText(i->text(0));
	mKeyPunct->setText(i->text(1));
}

void CIRKSettingsForm::changedFastListSelection(QListViewItem *i)
{
	m_pCurrentFastListItem=i;
	mKeyUnFast->setText(i->text(0));
	mKeyFast->setText(i->text(1));
}


void CIRKSettingsForm::changedKeyboard(int n)
{
	CIRKeyboard *pKeyboard=m_pIRK->currentKeyboard(n);
#if !POCKETOP_BUILD	
	mKeyboard->setCurrentItem(n);
#endif
	QString *pShiftMap=pKeyboard->shiftMapArray();
	QString *pPunctMap=pKeyboard->punctMapArray();
	QString *pCmdMap=pKeyboard->cmdMapArray();
	QString *pNumMap=pKeyboard->numMapArray();
	QString *pFastMap=pKeyboard->fastMapArray();
	int k;
	char key[2];
	key[1]='\0';
	mShiftKeyListView->clear();
	mNumKeyListView->clear();
	mCmdKeyListView->clear();
	mPunctKeyListView->clear();
	mFastKeyListView->clear();
	for(k=0; k<256; k++)
	{
		key[0]=k;
		if(pShiftMap[k])
		{
			QListViewItem *pItem=new QListViewItem(mShiftKeyListView,((k<=30)?CGIRK::nameForSpecial(k):key),pShiftMap[k]);
		}
		if(pNumMap[k])
		{
			QListViewItem *pItem=new QListViewItem(mNumKeyListView,((k<=30)?CGIRK::nameForSpecial(k):key),pNumMap[k]);
		}
		if(pCmdMap[k])
		{
			QListViewItem *pItem=new QListViewItem(mCmdKeyListView,((k<=30)?CGIRK::nameForSpecial(k):key),pCmdMap[k]);
		}
		if(pPunctMap[k])
		{
			QListViewItem *pItem=new QListViewItem(mPunctKeyListView,((k<=30)?CGIRK::nameForSpecial(k):key),pPunctMap[k]);
		}
		if(pFastMap[k])
		{
			QListViewItem *pItem=new QListViewItem(mFastKeyListView,((k<=30)?CGIRK::nameForSpecial(k):key),pFastMap[k]);
		}
	}
}

void CIRKSettingsForm::close()
{
	done(QDialog::Rejected);
}

void CIRKSettingsForm::selectShiftKey()
{
	getKey(mKeyUnShifted,mShiftKeyButton);
}

void CIRKSettingsForm::selectNumKey()
{
	getKey(mKeyUnNum,mNumKeyButton);
}

void CIRKSettingsForm::selectCmdKey()
{
	getKey(mKeyUnCmd,mCmdKeyButton);
}

void CIRKSettingsForm::selectPunctKey()
{
	getKey(mKeyUnPunct,mPunctKeyButton);
}

void CIRKSettingsForm::selectFastKey()
{
	getKey(mKeyUnFast,mFastKeyButton);
}

void CIRKSettingsForm::getKey(QLineEdit *pTarget,QPushButton *pTrigger)
{
	if(m_pKeyTarget==NULL)
	{
		m_pKeyTarget=pTarget;
		m_pKeyTrigger=pTrigger;
		m_pIRK->setRaw(true);
		grabKeyboard();
	}
}

void CIRKSettingsForm::keyPressEvent(QKeyEvent *e)
{
	if(m_pKeyTarget)
	{
		QString s;
		if(isprint(e->key()))
		{
			s.sprintf("%c",e->key());
		}else{
			s.sprintf("%s",CGIRK::nameForSpecial(e->key()));
		}
		m_pKeyTarget->setText(s);
		m_pKeyTrigger->toggle();
		m_pKeyTarget=NULL;
		m_pKeyTrigger=NULL;
		m_pIRK->setRaw(false);
		releaseKeyboard();
	}
}

void CIRKSettingsForm::enableDisableRepeat(bool enable)
{
	if(enable)
	{
		mRepeatPeriod->setEnabled(true);
		mRepeatDelay->setEnabled(true);
	}else{
		mRepeatPeriod->setEnabled(false);
		mRepeatDelay->setEnabled(false);
	}
}
