#include "irk.h"


#include <qpixmap.h>
#include <stdio.h>
#if !(X86)
#include <lirc_client.h>
#endif
#include <stdlib.h>
#include <unistd.h>
#include <qsocketnotifier.h>
#include <qradiobutton.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include "CIRKSettingsForm.h"
#include <qpushbutton.h>
#include <qpe/config.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <qpe/global.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <qpe/qcopenvelope_qws.h>
#include <termios.h>
#include <qlistview.h>
#include "irkimpl.h"
#include <qpe/qpeapplication.h>
#include <ctype.h>

extern const char *Infrared_xpm[];
extern const char *pocketop_xpm[];
extern const char *targus_xpm[];

unsigned short CGIRK::m_pocketopLookupTable[128];
unsigned short CGIRK::m_targusLookupTable[128];
/** Table of IrDA device names for various Zaurii - indexed as in the ZAURUS_MODEL enumerated type */
const char *CGIRK::m_irdaDeviceNames[ZAURUS_MODEL_MAX]={
	"/dev/ttyS2", // SL5000,SL5500
	"/dev/ttyS1" // SL5600,SLC7xx
};

CGIRK::CGIRK( QWidget * parent, const char * name, WFlags f) 
 : CGIRKBase( parent, name, f )
{
	// setup a timer to cancel shift key's
	m_shiftTimer=new QTimer(this);
	connect(m_shiftTimer,SIGNAL(timeout()),SLOT(cancelShift()));
	// setup to pickup end of packet
	m_repeatTimer=new QTimer(this);
	connect(m_repeatTimer,SIGNAL(timeout()),SLOT(repeat()));
	// default modifier and caps settings
	m_capsLock=m_numLock=m_isShifted=m_isNum=m_isPunct=m_isCmd=m_isCtrl=m_isAlt=false;
	// we haven't opened the keyboard yet
	m_keyboardOpen=false;
	// auto deletion in keyboard list
	m_keyboardList.setAutoDelete( TRUE );
	m_lastDown1=m_lastDown2=0;
	m_lastMod=0;
	
	// not in raw mode initially
	m_rawMode=false;
		
	// create our little status bar (show when you click on the input method icon in the toolbar)
	connect(m_on,SIGNAL(toggled(bool)),SLOT(on(bool)));
	connect(m_off,SIGNAL(toggled(bool)),SLOT(off(bool)));
	m_caps->setChecked(false);
	m_num->setChecked(false);
	connect(m_caps,SIGNAL(toggled(bool)),SLOT(caps(bool)));
	connect(m_num,SIGNAL(toggled(bool)),SLOT(num(bool)));
	connect(m_settingsButton,SIGNAL(clicked()),SLOT(settings()));
	
	// Create the settings dialog
	pSettingsForm=new CIRKSettingsForm(this,0,true);
	
	// Create the task selector
	m_pTaskSelector=new TaskSelector(this);
	// Create the task switcher
	m_pTaskSwitcher=new TaskSwitcher();
	
	// read the config file
	readConfig();
	
	// update our current status from the defaults in the config file
	m_on->setChecked(pSettingsForm->mDefaultOn->isChecked());
	m_off->setChecked(pSettingsForm->mDefaultOff->isChecked());

	Config config("qpe");
	config.setGroup("Screensaver");
	m_lightlevel = config.readNumEntry("Brightness",255);
	if (m_lightlevel==0) {
		m_lightlevel = 255;
		m_lighton = false;
	}
	
	// get initial rotation
	QString d = getenv("QWS_DISPLAY");
	m_rotation=0;
	if ( d.contains("Rot90") ) { // No tr
		m_rotation = 90;
	} else if ( d.contains("Rot180") ) { // No tr
		m_rotation = 180;
	} else if ( d.contains("Rot270") ) { // No tr
		m_rotation = 270;
	}
	m_isRotated=false;
}

CGIRK::~CGIRK()
{
	keyboardClose();
}

CIRKeyboard *CGIRK::currentKeyboard(int n)
{
	if(n==-1)
		return m_currentKeyboard;
	else
		return m_keyboardList.at(n);
}

void CGIRK::lircOpen()
{
#if !(X86)
	// open a socket to lircd
	fd=lirc_init("irk",1);
	if(fd!=-1)
	{
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read );
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailableLIRC(int)));
			m_keyboardOpen=true;
		}else{
			lircClose();
		}
	}
#endif
}

void CGIRK::lircClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the link to lircd
		lirc_deinit();
		fd=-1;
	}
#endif
}

void CGIRK::targusInit()
{
	memset(m_targusLookupTable,0,sizeof(m_targusLookupTable));
	m_pocketopLookupTable[0xf]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0xd]='=';
	m_pocketopLookupTable[0x2b]='\\';
	m_pocketopLookupTable[0x35]='/';
	m_pocketopLookupTable[0x29]='`';
	m_pocketopLookupTable[0x1a]='[';
	m_pocketopLookupTable[0x1b]=']';
	m_pocketopLookupTable[0x65]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x2]='1';
	m_pocketopLookupTable[0x3]='2';
	m_pocketopLookupTable[0x4]='3';
	m_pocketopLookupTable[0x5]='4';
	m_pocketopLookupTable[0x6]='5';
	m_pocketopLookupTable[0x7]='6';
	m_pocketopLookupTable[0x8]='7';
	m_pocketopLookupTable[0x9]='8';
	m_pocketopLookupTable[0xa]='9';
	m_pocketopLookupTable[0xb]='0';
	m_pocketopLookupTable[0x10]='q';
	m_pocketopLookupTable[0x11]='w';
	m_pocketopLookupTable[0x12]='e';
	m_pocketopLookupTable[0x13]='r';
	m_pocketopLookupTable[0x14]='t';
	m_pocketopLookupTable[0x15]='y';
	m_pocketopLookupTable[0x16]='u';
	m_pocketopLookupTable[0x17]='i';
	m_pocketopLookupTable[0x18]='o';
	m_pocketopLookupTable[0x19]='p';
	m_pocketopLookupTable[0xc]='-';
	m_pocketopLookupTable[0xe]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x3b]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x3c]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x3d]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x3e]=CGIRK_KEY_MEMO;
	m_pocketopLookupTable[0x3a]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x1e]='a';
	m_pocketopLookupTable[0x1f]='s';
	m_pocketopLookupTable[0x20]='d';
	m_pocketopLookupTable[0x21]='f';
	m_pocketopLookupTable[0x22]='g';
	m_pocketopLookupTable[0x23]='h';
	m_pocketopLookupTable[0x24]='j';
	m_pocketopLookupTable[0x25]='k';
	m_pocketopLookupTable[0x26]='l';
	m_pocketopLookupTable[0x27]=';';
	m_pocketopLookupTable[0x28]='\'';
	m_pocketopLookupTable[0x1c]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x2a]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x2c]='z';
	m_pocketopLookupTable[0x2d]='x';
	m_pocketopLookupTable[0x2e]='c';
	m_pocketopLookupTable[0x2f]='v';
	m_pocketopLookupTable[0x30]='b';
	m_pocketopLookupTable[0x31]='n';
	m_pocketopLookupTable[0x32]='m';
	m_pocketopLookupTable[0x33]=',';
	m_pocketopLookupTable[0x34]='.';
	m_pocketopLookupTable[0x48]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x36]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x1d]=CGIRK_KEY_CTRL;
	// m_pocketopLookupTable[0x]=CGIRK_KEY_PUNCT;
	m_pocketopLookupTable[0x62]=CGIRK_KEY_CMD;
	// m_pocketopLookupTable[0x]=CGIRK_KEY_NUM;
	m_pocketopLookupTable[0x38]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x39]=' ';
	m_pocketopLookupTable[0x63]=' ';
	m_pocketopLookupTable[0x68]=' ';
	m_pocketopLookupTable[0x61]=CGIRK_KEY_FUNCTION;
	m_pocketopLookupTable[0x4b]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x50]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x4d]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_DEL;
	m_pocketopLookupTable[0x1]=CGIRK_KEY_ESC;
	m_pocketopLookupTable[0x68]=CGIRK_KEY_CONFIRM;
	m_pocketopLookupTable[0x64]=CGIRK_KEY_HOME;

}

void CGIRK::pocketopInit()
{
	memset(m_pocketopLookupTable,0,sizeof(m_pocketopLookupTable));
	m_pocketopLookupTable[0x69]=CGIRK_KEY_TAB;
	m_pocketopLookupTable[0x01]='q';
	m_pocketopLookupTable[0x09]='w';
	m_pocketopLookupTable[0x11]='e';
	m_pocketopLookupTable[0x19]='r';
	m_pocketopLookupTable[0x21]='t';
	m_pocketopLookupTable[0x29]='y';
	m_pocketopLookupTable[0x31]='u';
	m_pocketopLookupTable[0x39]='i';
	m_pocketopLookupTable[0x41]='o';
	m_pocketopLookupTable[0x49]='p';
	m_pocketopLookupTable[0x51]='-';
	m_pocketopLookupTable[0x59]=CGIRK_KEY_BACKSPACE;
	m_pocketopLookupTable[0x40]=CGIRK_KEY_DATE;
	m_pocketopLookupTable[0x48]=CGIRK_KEY_ADDRESS;
	m_pocketopLookupTable[0x50]=CGIRK_KEY_TODO;
	m_pocketopLookupTable[0x58]=CGIRK_KEY_MEMO;
	m_pocketopLookupTable[0x77]=CGIRK_KEY_CAPS;
	m_pocketopLookupTable[0x07]='a';
	m_pocketopLookupTable[0x0f]='s';
	m_pocketopLookupTable[0x17]='d';
	m_pocketopLookupTable[0x1f]='f';
	m_pocketopLookupTable[0x27]='g';
	m_pocketopLookupTable[0x2f]='h';
	m_pocketopLookupTable[0x32]='j';
	m_pocketopLookupTable[0x3a]='k';
	m_pocketopLookupTable[0x42]='l';
	m_pocketopLookupTable[0x4a]=';';
	m_pocketopLookupTable[0x52]='\'';
	m_pocketopLookupTable[0x5a]=CGIRK_KEY_RETURN;
	m_pocketopLookupTable[0x65]=CGIRK_KEY_SHIFTL;
	m_pocketopLookupTable[0x03]='z';
	m_pocketopLookupTable[0x0b]='x';
	m_pocketopLookupTable[0x13]='c';
	m_pocketopLookupTable[0x1b]='v';
	m_pocketopLookupTable[0x23]='b';
	m_pocketopLookupTable[0x33]='n';
	m_pocketopLookupTable[0x3b]='m';
	m_pocketopLookupTable[0x43]=',';
	m_pocketopLookupTable[0x4b]='.';
	m_pocketopLookupTable[0x53]=CGIRK_KEY_UP;
	m_pocketopLookupTable[0x7d]=CGIRK_KEY_SHIFTR;
	m_pocketopLookupTable[0x6d]=CGIRK_KEY_CTRL;
	m_pocketopLookupTable[0x76]=CGIRK_KEY_PUNCT;
	m_pocketopLookupTable[0x75]=CGIRK_KEY_CMD;
	m_pocketopLookupTable[0x6e]=CGIRK_KEY_NUM;
	m_pocketopLookupTable[0x66]=CGIRK_KEY_ALT;
	m_pocketopLookupTable[0x2b]=' ';
	m_pocketopLookupTable[0x34]=' ';
	m_pocketopLookupTable[0x3c]=CGIRK_KEY_FUNCTION;
	m_pocketopLookupTable[0x44]=CGIRK_KEY_LEFT;
	m_pocketopLookupTable[0x4c]=CGIRK_KEY_DOWN;
	m_pocketopLookupTable[0x54]=CGIRK_KEY_RIGHT;
	m_pocketopLookupTable[0x5b]=CGIRK_KEY_DEL;
}

void CGIRK::targusOpen()
{
	// Init the pocketop lookup table
	targusInit();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePocketop(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)targus_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}


void CGIRK::pocketopOpen()
{
	// Init the pocketop lookup table
	pocketopInit();
#if !(X86)
	// open the IRDA serial port
	fd=open(m_irdaDeviceNames[m_zaurusModel],O_RDONLY|O_NONBLOCK|O_NOCTTY);
	if(fd!=-1)
	{
		struct termios options;
		// Setup the IR serial port
		tcgetattr(fd, &options);
		
		options.c_iflag = IGNBRK;
		options.c_cflag = (CS8|CREAD|CLOCAL|CRTSCTS);
		options.c_oflag = (NL0|CR0|TAB0|BS0|VT0|FF0);
		options.c_lflag = 0;
		
		// 9600 baud
		cfsetispeed(&options, B9600);
		cfsetospeed(&options, B9600);
		
		// Set the new options for the port...
		tcsetattr(fd, TCSANOW, &options);
	
		// setup a socket notifier to tell us data is available
		sn = new QSocketNotifier( fd, QSocketNotifier::Read);
		if(sn)
		{
			connect( sn, SIGNAL(activated(int)), SLOT(dataAvailablePocketop(int)));
			m_keyboardOpen=true;
		}else{
			::close(fd);
			fd=-1;
		}
	}else{
		QMessageBox mb("IRK","Couldn't open IrDA Serial Port.\nEither something else is using the Infrared, or you're using an OpenZaurus ROM with broken IrDA serial support.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
		mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
		mb.exec();
		m_on->setChecked(false);
	}
#endif
}

void CGIRK::targusClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}
void CGIRK::pocketopClose()
{
#if !(X86)
	if(fd!=-1)
	{
		// close the device
		::close(fd);
		fd=-1;
	}
#endif
}

void CGIRK::keyboardOpen()
{
	if(m_keyboardOpen==false)
	{
		switch(m_currentKeyboard->mode())
		{
			case IRKMODE_LIRC:
				lircOpen();
				break;
			case IRKMODE_POCKETOP:
				pocketopOpen();
				break;
			case IRKMODE_IRDA:
				break;
			case IRKMODE_TARGUS:
				targusOpen();
				break;
		}
	}
}

void CGIRK::keyboardClose()
{
	if(m_keyboardOpen==true)
	{
		if(sn)
		{
			// delete the socket notifier
			delete sn;
			sn=NULL;
		}
		switch(m_currentKeyboard->mode())
		{
			case IRKMODE_LIRC:
				lircClose();
				break;
			case IRKMODE_TARGUS:
				targusClose();
				break;
			case IRKMODE_POCKETOP:
				pocketopClose();
				break;
			case IRKMODE_IRDA:
				break;
		}
		m_keyboardOpen=false;
	}
}

void CGIRK::readConfig()
{
	// clear the keyboard list ready for the new one to be loaded in
	m_keyboardList.clear();
#if !POCKETOP_BUILD
	pSettingsForm->mKeyboard->clear();
#endif
	Config config("IRK");
	config.setGroup("Config");
	
	bool defOn=config.readBoolEntry ( "DefaultOn", false);
	
	pSettingsForm->mDefaultOn->setChecked(defOn);
	pSettingsForm->mDefaultOff->setChecked(!defOn);
	
	// read number of keyboards supports
	int nKeyboards=config.readNumEntry("NumKeyboards",0);

	// read key repeat time
	m_keyRepeatEnable=config.readBoolEntry("KeyRepeatEnable",true);
	pSettingsForm->mRepeatEnable->setChecked(m_keyRepeatEnable);
	m_keyRepeatPeriod=config.readNumEntry("KeyRepeatPeriod",500);
	pSettingsForm->mRepeatPeriod->setValue(m_keyRepeatPeriod);
	m_keyRepeatDelay=config.readNumEntry("KeyRepeatDelay",500);
	pSettingsForm->mRepeatDelay->setValue(m_keyRepeatDelay);

	// Read zaurus model (as IrDA serial port is a different device on SL5500 vs Sl5600)
	m_zaurusModel=(ZAURUS_MODEL)config.readNumEntry("ZaurusModel",0);
	pSettingsForm->mZaurusModel->setCurrentItem(m_zaurusModel);
	
	// read names of keyboards
	QString keyboardName,kEntry;
	int k;
	for(k=0; k<nKeyboards; k++)
	{
		kEntry.sprintf("Keyboard[%d]",k);
		keyboardName=config.readEntry(kEntry);
		m_keyboardList.append(new CIRKeyboard(keyboardName));
	}
	
	int defaultKeyboardID=config.readNumEntry("DefaultKeyboard",0);
	m_currentKeyboard=m_keyboardList.at(defaultKeyboardID);
	
	// read config for each keyboard
	for(k=0; k<nKeyboards; k++)
	{
		config.setGroup(m_keyboardList.at(k)->name());
		QString mode=config.readEntry("Mode","LIRC");
		if(mode=="LIRC")
		{
			m_keyboardList.at(k)->setMode(IRKMODE_LIRC);
			m_keyboardList.at(k)->setLIRCConf(config.readEntry("LIRC_Config",""));
		}else if(mode=="POCKETOP"){
			m_keyboardList.at(k)->setMode(IRKMODE_POCKETOP);
		}else if(mode=="TARGUS"){
			m_keyboardList.at(k)->setMode(IRKMODE_TARGUS);
		}else{
			m_keyboardList.at(k)->setMode(IRKMODE_IRDA);
		}
#if !POCKETOP_BUILD
		if (m_keyboardList.at(k)->mode() == IRKMODE_POCKETOP)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)pocketop_xpm), m_keyboardList.at(k)->name());
		else if (m_keyboardList.at(k)->mode() == IRKMODE_TARGUS)
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)targus_xpm), m_keyboardList.at(k)->name());
		else	
			pSettingsForm->mKeyboard->insertItem(QPixmap((const char **)Infrared_xpm), m_keyboardList.at(k)->name());
#endif
		int numShifted=config.readNumEntry("NumShifted",0);
		m_keyboardList.at(k)->setNumShifted(numShifted);
		int numPunct=config.readNumEntry("NumPunct",0);
		m_keyboardList.at(k)->setNumPunct(numPunct);
		int numCmd=config.readNumEntry("NumCmd",0);
		m_keyboardList.at(k)->setNumCmd(numCmd);
		int numNum=config.readNumEntry("NumNum",0);
		m_keyboardList.at(k)->setNumNum(numNum);
		int numFast=config.readNumEntry("NumFast",0);
		m_keyboardList.at(k)->setNumFast(numFast);
		int n,sep;
		QString code,sEntry;
		QChar key;
		for(n=0; n<numShifted; n++)
		{
			sEntry.sprintf("S[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setShiftMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numPunct; n++)
		{
			sEntry.sprintf("P[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setPunctMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numCmd; n++)
		{
			sEntry.sprintf("C[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setCmdMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numNum; n++)
		{
			sEntry.sprintf("N[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setNumMapEntry(key,code.mid(sep+1));
		}
		for(n=0; n<numFast; n++)
		{
			sEntry.sprintf("F[%d]",n);
			code=config.readEntry(sEntry,"0:#0");
			sep=code.find(':');
			key=(QChar)code.left(sep).toInt();
			m_keyboardList.at(k)->setFastMapEntry(key,code.mid(sep+1));
		}
	}
	
	pSettingsForm->changedKeyboard(defaultKeyboardID);
#if !POCKETOP_BUILD
	pSettingsForm->mKeyboard->setCurrentItem(m_keyboardList.find(m_currentKeyboard));
#endif
}

void CGIRK::writeConfig()
{
	Config config("IRK");
	config.setGroup("Config");
	config.writeEntry ( "DefaultOn", pSettingsForm->mDefaultOn->isChecked());
	int nKeyboards=m_keyboardList.count();
	config.writeEntry("NumKeyboards",nKeyboards);
	config.writeEntry("KeyRepeatEnable",m_keyRepeatEnable);
	config.writeEntry("KeyRepeatPeriod",m_keyRepeatPeriod);
	config.writeEntry("KeyRepeatDelay",m_keyRepeatDelay);
	config.writeEntry("ZaurusModel",(int)m_zaurusModel);
	int k;
	for(k=0; k<nKeyboards; k++)
	{
		QString kEntry;
		kEntry.sprintf("Keyboard[%d]",k);
		config.writeEntry(kEntry,m_keyboardList.at(k)->name());
	}
	config.writeEntry("DefaultKeyboard",m_keyboardList.find(m_currentKeyboard));
	for(k=0; k<nKeyboards; k++)
	{
		config.setGroup(m_keyboardList.at(k)->name());
		if(m_keyboardList.at(k)->mode()==IRKMODE_LIRC)
		{
			config.writeEntry("Mode","LIRC");
			config.writeEntry("LIRC_Config",m_keyboardList.at(k)->lircConf());
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_IRDA){
			config.writeEntry("Mode","IRDA");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_POCKETOP){
			config.writeEntry("Mode","POCKETOP");
		}else if(m_keyboardList.at(k)->mode()==IRKMODE_TARGUS){
			config.writeEntry("Mode","TARGUS");
		}
		config.writeEntry("NumShifted",m_keyboardList.at(k)->numShifted());
		config.writeEntry("NumCmd",m_keyboardList.at(k)->numCmd());
		config.writeEntry("NumPunct",m_keyboardList.at(k)->numPunct());
		config.writeEntry("NumNum",m_keyboardList.at(k)->numNum());
		config.writeEntry("NumFast",m_keyboardList.at(k)->numFast());
		int n,count;
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->shiftMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("S[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->shiftMapEntry(n)));
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->cmdMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("C[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->cmdMapEntry(n)));
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->punctMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("P[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->punctMapEntry(n)));
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->numMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("N[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->numMapEntry(n)));
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
		count=0;
		for(n=0; n<256; n++)
		{
			if(m_keyboardList.at(k)->fastMapEntry(n).length())
			{
				QString sEntry;
				QString sCode;
				sEntry.sprintf("F[%d]",count);
				sCode.sprintf("%d:%s",n,(const char*)(m_keyboardList.at(k)->fastMapEntry(n)));
				config.writeEntry(sEntry,sCode);
				count++;
			}
		}
	}
}

void CGIRK::settings()
{
	pSettingsForm->showMaximized();
	
	if(pSettingsForm->exec()==QDialog::Accepted)
	{
		bool isOpen=m_keyboardOpen;
		keyboardClose();
#if !POCKETOP_BUILD
		int newKeyboardId=pSettingsForm->mKeyboard->currentItem();
		m_currentKeyboard=m_keyboardList.at(newKeyboardId);
#endif
		m_keyRepeatPeriod=pSettingsForm->mRepeatPeriod->value();
		m_keyRepeatDelay=pSettingsForm->mRepeatDelay->value();
		m_keyRepeatEnable=pSettingsForm->mRepeatEnable->isChecked();
		m_zaurusModel=(ZAURUS_MODEL)pSettingsForm->mZaurusModel->currentItem();
		m_currentKeyboard->resetMaps();
		
		QListViewItem *pI;
		for(pI=pSettingsForm->mShiftKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setShiftMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumShifted(m_currentKeyboard->numShifted()+1);
			}
		}
		for(pI=pSettingsForm->mNumKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setNumMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumNum(m_currentKeyboard->numNum()+1);
			}
		}
		for(pI=pSettingsForm->mCmdKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setCmdMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumCmd(m_currentKeyboard->numCmd()+1);
			}
		}
		for(pI=pSettingsForm->mPunctKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setPunctMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumPunct(m_currentKeyboard->numPunct()+1);
			}
		}
		for(pI=pSettingsForm->mFastKeyListView->firstChild(); pI; pI=pI->nextSibling())
		{
			QChar ch=specialForName(pI->text(0));
			if(ch)
			{
				m_currentKeyboard->setFastMapEntry(ch,pI->text(1));
				m_currentKeyboard->setNumFast(m_currentKeyboard->numFast()+1);
			}
		}
		
		if(isOpen)
			keyboardOpen();
		writeConfig();
	}
}


void CGIRK::cancelShift()
{
	m_shiftTimer->stop();
	m_isShifted=false;
	emit key(0,Key_Shift,0,false,0);
}

void CGIRK::setRepeat(int ms,int delay)
{
	m_keyRepeatPeriod=ms;
	if(delay>0)
		m_keyRepeatDelay=delay;
}


void CGIRK::caps(bool state)
{
	m_capsLock=state;
}

void CGIRK::num(bool state)
{
	m_numLock=state;
}

void CGIRK::on(bool state)
{
	if(state)
	{
		keyboardOpen();
	}
}

void CGIRK::off(bool state )
{
	if(state)
	{
		keyboardClose();
		m_lastDown1=m_lastDown2=0;
		m_repeatTimer->stop();
		m_caps->setChecked(false);
		m_num->setChecked(false);
		m_capsLock=m_numLock=m_isShifted=m_isNum=m_isPunct=m_isCmd=m_isCtrl=m_isAlt=false;
	}
}


QSize CGIRK::sizeHint() const
{
    return QSize(320,height());
}

QString CGIRK::getShifted(char c)
{
	return m_currentKeyboard->shiftMapEntry(c);
}

QString CGIRK::getNum(char c)
{
	return m_currentKeyboard->numMapEntry(c);
}

QString CGIRK::getPunct(char c)
{
	return m_currentKeyboard->punctMapEntry(c);
}

QString CGIRK::getCmd(char c)
{
	return m_currentKeyboard->cmdMapEntry(c);
}

QString CGIRK::getFast(char c)
{
	return m_currentKeyboard->fastMapEntry(c);
}

bool CGIRK::isNum()
{
	return m_isNum||m_numLock;
}

void CGIRK::handleKeyPress(unsigned short c,bool down)
{

	QString macro;
	
	m_repeatTimer->stop();
	
	if(c<=30)
	{
		switch(c)
		{
			case CGIRK_KEY_SHIFTL:
			case CGIRK_KEY_SHIFTR:
				m_isShifted=down;
				emit key(0xFFFF,Key_Shift,m_lastMod,down,0);
				if(!(m_lastMod&(Qt::ControlButton|Qt::AltButton)))
					c=0;
				break;

			case CGIRK_KEY_CTRL:
				m_isCtrl=down;
				emit key(0xFFFF,Key_Control,m_lastMod,down,0);
				c=0;
				break;

			case CGIRK_KEY_PUNCT:
				m_isPunct=down;
				c=0;
				break;

			case CGIRK_KEY_CMD:
				m_isCmd=down;
				c=0;
				break;

			case CGIRK_KEY_NUM:
				m_isNum=down;
				emit key(0xFFFF,Key_NumLock,m_lastMod,down,0);
				if(down)
					m_lastDown2=Key_NumLock;
				else if(m_lastDown2==Key_NumLock)
				{
					m_numLock=!m_numLock;
					m_num->setChecked(m_numLock);
					m_lastDown2=0xFFFF;
				}
				c=0;
				break;

			case CGIRK_KEY_ALT:
				m_isAlt=down;
				emit key(0xFFFF,Key_Alt,m_lastMod,down,0);
				c=0;
				break;

			case CGIRK_KEY_HOME:
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
					break;
			case CGIRK_KEY_LEFT:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_Home,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Home);
					}
				}else if(isNum()){
					emit key('.','.',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('.','.');
					}
				}else{
					emit key(0xFFFF,Key_Left,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Left);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_RIGHT:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_End,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_End);
					}
				}else if(isNum()){
					emit key('=','=',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('=','=');
					}
				}else{
					emit key(0xFFFF,Key_Right,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Right);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_UP:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_PageUp,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageUp);
					}
				}else if(isNum()){
					emit key('-','-',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('-','-');
					}
				}else{
					emit key(0xFFFF,Key_Up,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Up);
					}
				}
				c=0;
				break;
			case CGIRK_KEY_DOWN:
				if(m_isCmd)
				{
					emit key(0xFFFF,Key_PageDown,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_PageDown);
					}
				}else if(isNum()){
					emit key('+','+',m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat('+','+');
					}
				}else{
					emit key(0xFFFF,Key_Down,m_lastMod,down,0);
					if(down)
					{
						startKeyRepeat(0xFFFF,Key_Down);
					}
				}
				c=0;
				break;
				
								
			default:
				{
					if(!(m_isNum|m_isAlt|m_isCmd))
					{
						// stanard key behaviours (no modifiers)
						switch(c)
						{
							case CGIRK_KEY_ESC:
								emit key(0xFFFF,Key_Escape,m_lastMod,down,0);
								if(down)
								{
									startKeyRepeat(0xFFFF,Key_Escape);
								}
								c=0;
								break;
							
							case CGIRK_KEY_CAPS:
								// toggle the caps lock bit
								if(down)
								{
									m_capsLock=!m_capsLock;
									m_caps->setChecked(m_capsLock);
								}
								emit key(0xFFFF,Key_CapsLock,m_lastMod,down,0);
								c=0;
								break;

							case CGIRK_KEY_FUNCTION:
								if(isNum())
									emit key('0','0',m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F12,m_lastMod,down,0);
								c=0;
								break;

							case CGIRK_KEY_DEL:
								emit key(0xFFFF,Key_Delete,m_lastMod,down,0);
								if(down)
								{
									startKeyRepeat(0xFFFF,Key_Delete);
								}
								c=0;
								break;
							case CGIRK_KEY_TAB:
								if(m_isCmd)
									emit key(0xFFFF,Key_Escape,m_lastMod,down,0);
								else
									emit key('\t',Key_Tab,m_lastMod,down,0);
								if(down)
								{
									startKeyRepeat('\t',Key_Tab);
								}
								c=0;
								break;
							case CGIRK_KEY_BACKSPACE:
								if(m_isCmd)
								{
									// CMD+Backspace does suspend, so send F34 as that's the code sent when you hold down the cancel button
									emit key(0xFFFF,Key_F34,m_lastMod,down,0);
								}else{
									emit key(0xFFFF,Key_Backspace,m_lastMod,down,0);
									if(down)
										startKeyRepeat(0xFFFF,Key_Backspace);
								}
								c=0;
								break;
							case CGIRK_KEY_CONFIRM:
									emit key(0xFFFF,Key_F33,m_lastMod,down,0);
									if(down)
									{
										startKeyRepeat(0xFFFF,Key_F33);
									}
								c=0;
								break;
							case CGIRK_KEY_RETURN:
								if(m_isCmd)
								{
									// CMD+Enter==OK, send F33 as Zaurus uses that as the OK button
									emit key(0xFFFF,Key_F33,m_lastMod,down,0);
								}else{
									emit key(0xFFFF,Key_Return,m_lastMod,down,0);
									if(down)
									{
										startKeyRepeat(0xFFFF,Key_Return);
									}
								}
								c=0;
								break;
				/*			case CGIRK_KEY_DATE:
								if(m_isCmd)
									emit key(0xFFFF,Key_F11,m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F9,m_lastMod,down,0);
								break;
							case CGIRK_KEY_ADDRESS:
								if(m_isCmd)
									emit key(0xFFFF,Key_Menu,m_lastMod,down,0);
								else
									emit key(0xFFFF,Key_F10,m_lastMod,down,0);
								break;
							case CGIRK_KEY_TODO:        
								if(down)
								{
									if(m_isCmd)
										Global::execute("calculator");
									else
										Global::execute("todolist");
								}
								break;
							case CGIRK_KEY_MEMO:
								if(down)
									Global::execute("textedit");*/
								break;
							default:
								break;
						}
					}
				}
				break;
		}
		m_lastMod=modifiers();
	}
	
	if(c)
	{
		if(((c>='a')&&(c<='z'))&&(!(isNum()|m_isPunct|m_isCmd)))
		{
			if(m_capsLock)
				c&=~32;
			if(m_isShifted)
				c^=32;
		}else{
			if((c==' ')&&(down)&&(m_isCmd)&&(m_isShifted))
			{
				c=0;
				m_isRotated=!m_isRotated;

				if(m_isRotated)
				{
					QPEApplication::setDefaultRotation((m_rotation==180)?270:180);
					QMessageBox mb("IRK","Rotation: On",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
					mb.exec();
				}else{
					QPEApplication::setDefaultRotation(m_rotation);
					QMessageBox mb("IRK","Rotation: Off.",QMessageBox::Warning,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
					mb.exec();
				}
				
			}else{
			
				if(isNum())
				{
					macro=getNum(c);
				}else if(m_isPunct){
					macro=getPunct(c);
				}else if(m_isShifted){
					macro=getShifted(c);
				}else if(m_isCmd){
					macro=getCmd(c);
				}else{
					macro=getFast(c);
				}
			
			}
			
			int l=macro.length();

			if(l)
			{
				if((l>1)&&(macro[0]=='#'))
				{
					// Numeric macro
					if(macro[1]=='#')
					{
						// escaped numeric macro
						c='#';
					}else{
						// convert numeric  from number to character...
						macro=macro.mid(1);
						c=macro.toUInt();
					}
				}else if((l>1)&&(macro[0]==':')&&(down)){
					// QCOP macro
					if(macro[1]==':')
					{
						// escaped QCOP macro
						c=':';
					}else{
						// extract QCOP stuff from macro
						int i;
						macro=macro.mid(1);
						i=macro.find(":");
						QString channel=macro.left(i);
						macro=macro.mid(i+1);
						i=macro.find(":");
						QString service=macro.left(i);
						macro=macro.mid(i+1);
						QCopEnvelope e(channel.utf8(), service.utf8());
						QString arg;
						while(macro.length())
						{
							i=macro.find(":");
							if(i>=0)
							{
								arg=macro.left(i);
								macro=macro.mid(i+1);
								e << arg.utf8();
							}else{
								e << macro.utf8();
								break;
							}
						}
						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='*')&&(down)){
					// Execute macro
					if(macro[1]=='*')
					{
						// escaped execute macro
						c='*';
					}else{
						// execute command given in macro...
						macro=macro.mid(1);
						Global::execute(macro);
						c=0;
						m_lastDown1=m_lastDown2=0;
					}
				}else if((l>1)&&(macro[0]=='$')){
					if(down)
					{
						// Extension macro
						if(macro[1]=='$')
						{
							// escaped extension macro
							c='$';
						}else{
							// process the rest of the macro...
							macro=macro.mid(1);
							if(macro=="taskSwitch")
							{
								m_pTaskSwitcher->doTaskSwitch(true);
							}else if(macro=="taskSelector") {
								m_pTaskSelector->startSelector();
							}
							c=0;
							m_lastDown1=m_lastDown2=0;
						}
					}else{
						if(macro[1]=='$')
						{
							// escaped extension macro
							c='$';
						}else{
							if(macro=="taskSelector") 
							{
								m_pTaskSelector->endSelector();
							}
						
							c=0;
							m_lastDown1=m_lastDown2=0;
						}
					}
				}else{
					if(down)
					{
						int i=0,l=macro.length();
						QChar ch;
						if(l>1)
						{
							// macro is longer that a single key - emit it once, but don't key repeat
							while(i<l)
							{
								ch=macro[i++];
								emit key(ch,ch,m_lastMod,true,0);
								emit key(ch,ch,m_lastMod,false,0);
							}
							if((m_lastDown1==0xFFFF)&&(m_lastDown2==0xFFFF))
							{
								c=0;
								m_lastDown1=m_lastDown2=0;
							}
						}else{
							//  macro is single character - use normal keyboard handling to get key repeats and stuff...
							ch=macro[0];
							c=ch;
						}
					}
				}
			}
		}
	}
	
	if(c)
	{
		unsigned short c0;
		if((m_lastMod&Qt::ControlButton)&&(c>='a')&&(c<='z'))
		{
			c0=c+1-'a';
			c=c&~32;
		}else
			c0=c;
		if(c0>256)
			c0=0xFFFF;
		emit key(c0,c,m_lastMod,down,0);
		if(down)
		{
			// setup the key repeat
			startKeyRepeat(c0,c);
		}
	}

}


void CGIRK::dataAvailableLIRC(int fd)
{
	char buf[100],*parse;
	long l,i=0;
	bool isKeyUp=false;
	l=read(fd,buf,100);
	buf[l]='\0';
	if(l)
	{
		for(i=0; (i<l)&&(buf[i]!=' '); i++);
		if(buf[i]==' ')
		{
			for(i++; (i<l)&&(buf[i]!=' '); i++);
			if(buf[i]==' ')
			{
				i++;
				parse=&buf[i];
				for(i++; (i<l)&&(buf[i]!=' '); i++);
				if(buf[i]==' ')
				{
					buf[i]='\0';
					// translate named chars into proper chars
					if(!strcmp("space",parse)) 
					{
						parse[0]=' ';
						parse[1]='\0';
					}
					
					l=strlen(parse);
					
					if((parse[l-2]=='U')&&(parse[l-1]=='p'))
					{
						isKeyUp=true;
					}
					
					if(l==1)
					{
						handleKeyPress(parse[0],!isKeyUp);
					}
					
					// handle control keys
					if(!strncmp("shift",parse,5)) {
						if(isKeyUp)
						{
							emit key(0,Key_Shift,0,false,0);
							m_isShifted=false;
						}else{
							emit key(0,Key_Shift,0,true,m_isShifted);
							m_shiftTimer->start(SHIFT_TIMEOUT,true);
							m_isShifted=true;
						}
					}else if(!strcmp("pagehome",parse)) {
						emit key(0,Key_Home,0,true,0);
						emit key(0,Key_Home,0,false,0);
					}else if(!strcmp("pageend",parse)) {
						emit key(0,Key_End,0,true,0);
						emit key(0,Key_End,0,false,0);
					}else if(!strcmp("up",parse)) {
						emit key(0,Key_Up,0,true,0);
						emit key(0,Key_Up,0,false,0);
					}else if(!strcmp("down",parse)) {
						emit key(0,Key_Down,0,true,0);
						emit key(0,Key_Down,0,false,0);
					}else if(!strcmp("left",parse)) {
						emit key(0,Key_Left,0,true,0);
						emit key(0,Key_Left,0,false,0);
					}else if(!strcmp("right",parse)) {
						emit key(0,Key_Right,0,true,0);
						emit key(0,Key_Right,0,false,0);
					}else if(!strcmp("return",parse)) {
						emit key(0,Key_Return,0,true,0);
						emit key(0,Key_Return,0,false,0);
					}else if(!strcmp("pageup",parse)) {
						emit key(0,Key_PageUp,0,true,0);
						emit key(0,Key_PageUp,0,false,0);
					}else if(!strcmp("pagedown",parse)) {
						emit key(0,Key_PageDown,0,true,0);
						emit key(0,Key_PageDown,0,false,0);
					}else if(!strcmp("escape",parse)) {
						emit key(0,Key_Escape,0,true,0);
						emit key(0,Key_Escape,0,false,0);
					}else if(!strcmp("backspace",parse)) {
						emit key(0,Key_Backspace,0,true,0);
						emit key(0,Key_Backspace,0,false,0);
					}else if(!strcmp("tab",parse)) {
						emit key('\t',Key_Tab,0,true,0);
						emit key('\t',Key_Tab,0,false,0);
					}else if(!strcmp("caps",parse)) {
						emit key(0,Key_CapsLock,0,true,0);
						emit key(0,Key_CapsLock,0,false,0);
						// toggle the caps lock bit
						m_capsLock=!m_capsLock;
						m_caps->setChecked(m_capsLock);
					}else if(!strcmp("menu",parse)) {
						emit key(0,Key_Menu,0,true,0);
						emit key(0,Key_Menu,0,false,0);
					}else if(!strcmp("winl",parse)) {
						emit key(0,Key_Super_L,0,true,0);
						emit key(0,Key_Super_L,0,false,0);
					}else if(!strcmp("winr",parse)) {
						emit key(0,Key_Super_R,0,true,0);
						emit key(0,Key_Super_R,0,false,0);
					}else if(!strcmp("alt",parse)) {
						emit key(0,Key_Alt,0,true,0);
						emit key(0,Key_Alt,0,false,0);
					}else if(!strcmp("altgra",parse)) {
						emit key(0,Key_Hyper_R,0,true,0);
						emit key(0,Key_Hyper_R,0,false,0);
					}
				}
				
			}
		}
	}
}

unsigned short CGIRK::modifiers()
{
	unsigned short rtn=0;
	if(m_isCtrl)
		rtn|=Qt::ControlButton;
	if(m_isShifted)
		rtn|=Qt::ShiftButton;
	if(m_isAlt)
		rtn|=Qt::AltButton;
	return rtn;
}

void CGIRK::dataAvailableTargus(int fd)
{
	int l;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_targusLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			handleKeyPress(code,down);
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)targus_xpm));
			mb.exec();
#endif
		}
	}
}

void CGIRK::dataAvailablePocketop(int fd)
{
	int l;
	unsigned char c;
	l=read(fd,&c,1);
	if(l==1)
	{
		bool down=(c&0x80)?false:true;
		unsigned short code;
		code=m_pocketopLookupTable[c&0x7f];
		
		if(m_rawMode)
		{
			emit key((isprint(code)?code:0xFFFF),code,m_lastMod,down,0);
			return;
		}
		
		if(!down)
			m_repeatTimer->stop();

		if(code)
		{
			handleKeyPress(code,down);
		}else{
#if DEBUG_KEY_CODES
			QString msg;
			msg.sprintf("Unrecognised keycode 0x%x",c);
			QMessageBox mb("IRK",msg,QMessageBox::NoIcon,QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
			mb.setIconPixmap(QPixmap((const char **)pocketop_xpm));
			mb.exec();
#endif
		}
	}
}

void CGIRK::repeat()
{
	if(m_lastDown1)
	{
		m_repeatTimer->start(m_keyRepeatPeriod);
		emit key(m_lastDown1,m_lastDown2,m_lastMod,true,true);
	}else{
		m_repeatTimer->stop();
	}
}

void CGIRK::setRaw(bool f)
{
	m_rawMode=f;
}

bool CGIRK::raw()
{
	return m_rawMode;
}

void CGIRK::startKeyRepeat(unsigned short c0, unsigned short c1)
{
	m_lastDown1=c0; m_lastDown2=c1;
	if(m_keyRepeatEnable)
		m_repeatTimer->start(m_keyRepeatDelay);
}


QChar CGIRK::specialForName(const QString &name)
{
	QChar ch=0;
	if(name.length()>1)
	{
		if(name=="CAPS")
			ch=CGIRK_KEY_CAPS;
		else if(name=="SHIFTL")
			ch=CGIRK_KEY_SHIFTL;
		else if(name=="SHIFTR")
			ch=CGIRK_KEY_SHIFTR;
		else if(name=="TAB")
			ch=CGIRK_KEY_TAB;
		else if(name=="CTRL")
			ch=CGIRK_KEY_CTRL;
		else if(name=="PUNCT")
			ch=CGIRK_KEY_PUNCT;
		else if(name=="CMD")
			ch=CGIRK_KEY_CMD;
		else if(name=="NUM")
			ch=CGIRK_KEY_NUM;
		else if(name=="ALT")
			ch=CGIRK_KEY_ALT;
		else if(name=="FUNCTION")
			ch=CGIRK_KEY_FUNCTION;
		else if(name=="LEFT")
			ch=CGIRK_KEY_LEFT;
		else if(name=="RIGHT")
			ch=CGIRK_KEY_RIGHT;
		else if(name=="UP")
			ch=CGIRK_KEY_UP;
		else if(name=="DOWN")
			ch=CGIRK_KEY_DOWN;
		else if(name=="DEL")
			ch=CGIRK_KEY_DEL;
		else if(name=="TAB")
			ch=CGIRK_KEY_TAB;
		else if(name=="BACKSPACE")
			ch=CGIRK_KEY_BACKSPACE;
		else if(name=="RETURN")
			ch=CGIRK_KEY_RETURN;
		else if(name=="DATE")
			ch=CGIRK_KEY_DATE;
		else if(name=="ADDRESS")
			ch=CGIRK_KEY_ADDRESS;
		else if(name=="TODO")
			ch=CGIRK_KEY_TODO;
		else if(name=="MEMO")
			ch=CGIRK_KEY_MEMO;
	}else{
		ch=name[0];
	}
	return ch;

}

const char *CGIRK::nameForSpecial(int n)
{
	switch(n)
	{
		case CGIRK_KEY_CAPS:
			return "CAPS";
			break;
		case CGIRK_KEY_SHIFTL:
			return "SHIFTL";
			break;
		case CGIRK_KEY_SHIFTR:
			return "SHIFTR";
			break;
		case CGIRK_KEY_CTRL:
			return "CTRL";
			break;
		case CGIRK_KEY_PUNCT:
			return "PUNCT";
			break;
		case CGIRK_KEY_CMD:
			return "CMD";
			break;
		case CGIRK_KEY_NUM:
			return "NUM";
			break;
		case CGIRK_KEY_ALT:
			return "ALT";
			break;
		case CGIRK_KEY_FUNCTION:
			return "FUNCTION";
			break;
		case CGIRK_KEY_LEFT:
			return "LEFT";
			break;
		case CGIRK_KEY_RIGHT:
			return "RIGHT";
			break;
		case CGIRK_KEY_UP:
			return "UP";
			break;
		case CGIRK_KEY_DOWN:
			return "DOWN";
			break;
		case CGIRK_KEY_DEL:
			return "DEL";
			break;
		case CGIRK_KEY_TAB:
			return "TAB";
			break;
		case CGIRK_KEY_BACKSPACE:
			return "BACKSPACE";
			break;
		case CGIRK_KEY_RETURN:
			return "RETURN";
			break;
		case CGIRK_KEY_DATE:
			return "DATE";
			break;
		case CGIRK_KEY_ADDRESS:
			return "ADDRESS";
			break;
		case CGIRK_KEY_TODO:
			return "TODO";
			break;
		case CGIRK_KEY_MEMO:
			return "MEMO";
			break;
		default:
			return "???";
			break;
	}	
}

CIRKeyboard::CIRKeyboard(QString name)
{
	m_name=name;
/*	memset(m_shiftMap,0,sizeof(m_shiftMap));
	memset(m_cmdMap,0,sizeof(m_cmdMap));
	memset(m_numMap,0,sizeof(m_numMap));
	memset(m_punctMap,0,sizeof(m_punctMap));*/
}

CIRKeyboard::~CIRKeyboard()
{
}

void CIRKeyboard::setMode(IRKMODE mode)
{
	m_mode=mode;
}

void CIRKeyboard::setShiftMapEntry(unsigned char key,QString shifted)
{
	m_shiftMap[key]=shifted;
}

void CIRKeyboard::setPunctMapEntry(unsigned char key,QString shifted)
{
	m_punctMap[key]=shifted;
}

void CIRKeyboard::setCmdMapEntry(unsigned char key,QString shifted)
{
	m_cmdMap[key]=shifted;
}

void CIRKeyboard::setNumMapEntry(unsigned char key,QString shifted)
{
	m_numMap[key]=shifted;
}

void CIRKeyboard::setFastMapEntry(unsigned char key,QString shifted)
{
	m_fastMap[key]=shifted;
}

void CIRKeyboard::setLIRCConf(QString conf)
{
	m_lircConf=conf;
}

void CIRKeyboard::setNumShifted(int n)
{
	m_numShifted=n;
}

void CIRKeyboard::setNumNum(int n)
{
	m_numNum=n;
}

void CIRKeyboard::setNumCmd(int n)
{
	m_numCmd=n;
}

void CIRKeyboard::setNumPunct(int n)
{
	m_numPunct=n;
}

void CIRKeyboard::setNumFast(int n)
{
	m_numFast=n;
}


QString CIRKeyboard::name()
{
	return m_name;
}

QString CIRKeyboard::lircConf()
{
	return m_lircConf;
}

IRKMODE CIRKeyboard::mode()
{
	return m_mode;
}

QString CIRKeyboard::shiftMapEntry(unsigned char  c)
{
	return m_shiftMap[c];
}

QString CIRKeyboard::numMapEntry(unsigned char  c)
{
	return m_numMap[c];
}

QString CIRKeyboard::cmdMapEntry(unsigned char  c)
{
	return m_cmdMap[c];
}

QString CIRKeyboard::punctMapEntry(unsigned char  c)
{
	return m_punctMap[c];
}

QString CIRKeyboard::fastMapEntry(unsigned char  c)
{
	return m_fastMap[c];
}

QString *CIRKeyboard::shiftMapArray()
{
	return m_shiftMap;
}

QString *CIRKeyboard::numMapArray()
{
	return m_numMap;
}

QString *CIRKeyboard::cmdMapArray()
{
	return m_cmdMap;
}

QString *CIRKeyboard::punctMapArray()
{
	return m_punctMap;
}

QString *CIRKeyboard::fastMapArray()
{
	return m_fastMap;
}

int CIRKeyboard::numShifted()
{
	return m_numShifted;
}

int CIRKeyboard::numPunct()
{
	return m_numPunct;
}

int CIRKeyboard::numCmd()
{
	return m_numCmd;
}

int CIRKeyboard::numNum()
{
	return m_numNum;
}

int CIRKeyboard::numFast()
{
	return m_numFast;
}

void CIRKeyboard::resetMaps()
{
	int i; 
	for(i=0; i<256; i++)
	{
		m_cmdMap[i]="";
		m_numMap[i]="";
		m_shiftMap[i]="";
		m_punctMap[i]="";
		m_fastMap[i]="";
	}
	m_numFast=
	m_numNum=
	m_numCmd=
	m_numShifted=
	m_numPunct=0;
}

